#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'zhangyue'

from datetime import datetime
from django import template
# from bee_django_exam.utils import get_user_model, get_user_name
from bee_django_mission.exports import filter_local_datetime

register = template.Library()


# 求两个值的差的绝对值
@register.filter
def get_difference_abs(a, b):
    return abs(a - b)


# 求两个值的差的绝对值
@register.filter
def multiple(a, b):
    return a * b

@register.filter
def get_int(a):
    return int(a)

#
# # 本地化时间
@register.filter
def local_datetime(_datetime):
    return filter_local_datetime(_datetime)



# coding=utf-8
__author__ = 'zhangyue'
import os, datetime, urllib2, json
from django.core.management.base import BaseCommand, CommandError
from bee_django_mission.models import ContentType, MissionType, Line, Stage


class MissionTypeData():
    mission_type_list = [
        {
            "id": 0,
            'name': '直播时长',
            "field_name": "duration",
            "conditions": "status:1",
            "aggragate_type": 2,
            "comparison_type": 1,
            'timestamp_field': "start_time",
            "operator_type": 1
        },
        {
            "id": 1,
            'name': '转介人数',
            "field_name": "id",
            "conditions": "level:3",
            "aggragate_type": 1,
            "comparison_type": 1,
            'timestamp_field': "created_at",
            "operator_type": 0
        },
        {
            "id": 2,
            'name': '考级通过',
            "field_name": "id",
            "conditions": "status:1",
            "aggragate_type": 1,
            "comparison_type": 1,
            "timestamp_field": None,
            "operator_type": 0
        },
        {
            "id": 3,
            'name': '直播天数',
            "field_name": "id",
            "conditions": "status:1",
            "aggragate_type": 3,
            "comparison_type": 1,
            'timestamp_field': "start_time",
            "operator_type": 0
        },
        {
            "id": 4,
            'name': '完成课程课件',
            "field_name": "id",
            "conditions": "finished_at__isnull:False",
            "aggragate_type": 1,
            "comparison_type": 1,
            "timestamp_field": None,
            "operator_type": 0
        },

    ]
    content_type_list = [
        {
            "app_label": "bee_django_course",
            "model": 'UserLive',
            'user_field': 'user',
            "info": '课程-习琴记录表',
            'mission_list': [
                mission_type_list[0], mission_type_list[3]
            ]
        },
        {
            "app_label": "bee_django_crm",
            "model": 'PreUser',
            'user_field': 'referral_user',
            "info": 'crm-转介记录表',
            'mission_list': [
                mission_type_list[1]
            ]
        },
        {
            "app_label": "bee_django_exam",
            "model": 'UserExamRecord',
            'user_field': 'user',
            "info": '考级-考级记录表',
            'mission_list': [
                mission_type_list[2]
            ]
        },
        {
            "app_label": "bee_django_course",
            "model": 'UserCourseSection',
            'user_field': 'user_course__user',
            "info": '课程-学生学习课件表',
            'mission_list': [
                mission_type_list[4]
            ]
        },
    ]


class lineDate():
    line_list = [
        {"name": "里程碑",
         "line_type": 1,
         "auto_finish": 0,
         "auto_start": 1
         },
        {"name": "周任务",
         "line_type": 2,
         "auto_finish": 1,
         "auto_start": 0
         },
    ]


class Command(BaseCommand):
    def handle(self, *args, **options):
        self.init_mission_type()
        self.init_line()
        self.init_stage()

    def init_line(self):
        line_list = lineDate.line_list
        for line in line_list:
            self.save_line(line)
        return

    def save_line(self, line):
        name = line['name']
        line_type = line['line_type']
        auto_finish = line['auto_finish']
        auto_start = line['auto_start']
        try:
            l = Line.objects.get(name=name)

        except:
            l = Line()
            l.name = name
            l.line_type = line_type
            l.auto_finish = auto_finish
            l.auto_start = auto_start
            l.save()
        return l

    def init_stage(self):
        stage_name = '本周任务'
        try:
            l = Line.objects.get(line_type=2)
            pass
        except:
            return
        try:
            Stage.objects.get(name=stage_name)
            return
        except:
            pass
        stage = Stage()
        stage.name = stage_name
        stage.level = 1
        stage.line = l
        stage.save()
        return stage

    def init_mission_type(self):
        content_type_list = MissionTypeData.content_type_list
        for content_type in content_type_list:
            ct = self.save_content_type(content_type)
            # print(ct)
            mission_list = content_type['mission_list']
            for mission_type in mission_list:
                self.save_mission_type(mission_type, ct)
        return

    def save_content_type(self, content_type):
        app_label = content_type['app_label']
        model = content_type['model']
        user_field = content_type['user_field']
        info = content_type['info']
        try:
            ct = ContentType.objects.get(app_label=app_label, model=model)
        except:
            ct = ContentType()
            ct.app_label = app_label
            ct.model = model
            ct.user_field = user_field
            ct.info = info
            ct.save()
        return ct

    def save_mission_type(self, mission_type, ct):
        name = mission_type["name"]
        field_name = mission_type["field_name"]
        conditions = mission_type["conditions"]
        aggregate_type = mission_type["aggragate_type"]
        comparison_type = mission_type["comparison_type"]
        timestamp_field = mission_type["timestamp_field"]
        operator_type = mission_type["operator_type"]
        try:
            mt = MissionType.objects.get(name=name)
        except:
            mt = MissionType()
            mt.name = name
            mt.field_name = field_name
            mt.conditions = conditions
            mt.aggregate_type = aggregate_type
            mt.comparison_type = comparison_type
            mt.timestamp_field = timestamp_field
            mt.operator_type = operator_type
            mt.content_type = ct
            mt.save()
        return mt



#!/usr/bin/env python
# -*- coding:utf-8 -*-

__author__ = 'bee'

from django.dispatch import receiver
from django.db.models.signals import post_save
from .models import Stage, UserStage, UserMission, Mission, Line, UserLine
from .dt import get_current_week_range_datetime

# ============receiver============
# 创建user_line后，自动创建该用户对应的stage
@receiver(post_save, sender=UserLine)
def create_user_stage(sender, **kwargs):
    user_line = kwargs['instance']
    if kwargs['created']:
        user_line.check_add_user_stage(check=False)


# 创建user_stage后，自动创建该用户对应stage里的所有mission
@receiver(post_save, sender=UserStage)
def create_user_missions(sender, **kwargs):
    user_stage = kwargs['instance']
    if kwargs['created']:
        user_stage.check_add_user_mission(check=False)


# ====================
# 给某学生添加user_line，
# line_type:1-主线任务，2-周任务
# check：检查是否已有user_line，如果有，则不添加。默认检查
def check_add_user_line(user, line_type, check=True):
    if check:
        try:
            UserLine.objects.get(user=user, line__line_type=line_type)
            return None
        except:
            pass
    line = Line.objects.all().filter(line_type=line_type).first()
    u_l = UserLine()
    u_l.line = line
    u_l.user = user
    u_l.save()
    return u_l



# 则根据课件是否自动通过，开启下一user_stage
# manual是否为手动完成
# 返回下一个user_stage，没有为空
# def start_next_user_stage(user_stage, manual=False):
#     if user_stage.stutas in [0]:
#         return
#     # 如果任务线为不自动完成，则不更新
#     if not manual or not user_stage.user_line.line.auto_start:
#         return
#     user_line = user_stage.user_line
#     return user_line.check_add_user_stage()

# 获取学生的当周user_stage
def get_current_week_stage(user):
    _start_date, _end_date = get_current_week_range_datetime()
    try:
        user_stage = UserStage.objects.get(start_at=_start_date, end_at=_end_date, user_line__user=user)
        return user_stage
    except:
        return None

# ============
# 获取学生的任务线
def get_user_week_line(user):
    return _get_user_line(user, line_type=2)


def get_user_unlimited_line(user):
    return _get_user_line(user, line_type=1)


def _get_user_line(user, line_type):
    try:
        return UserLine.objects.get(user=user, line__line_type=line_type)

    except:
        return None


# ====================


# 更新user_mission为最新的stage中的mission

def change_user_mission(user_stage):
    stage = user_stage.stage
    user = user_stage.user_line.user
    mission_list = Mission.objects.filter(stage=stage)
    for mission in mission_list:
        # 检查重复，没有则创建
        try:
            UserMission.objects.get(mission=mission, user_stage=user_stage)
            continue
        except:
            user_mission = UserMission()
            user_mission.user = user
            user_mission.mission = mission
            user_mission.save()

# 给某学生添加周任务，check检查是否已有周任务，如果有，则不添加。默认检查
# def add_user_week_line(user, check=True):
#     if check:
#         try:
#             UserLine.objects.get(user=user, line__line_type=2)
#             return None
#         except:
#             pass
#     line = Line.objects.all().filter(line_type=2).first()
#     u_l = UserLine()
#     u_l.line = line
#     u_l.user = user
#     u_l.save()
#     return u_l

# def update_user_missions(user, stage_list=None):
#     # 没有指定stage，则更新学生所有未完成任务
#     if not stage_list:
#         user_stage_list = UserStage.objects.filter(user=user, finish_at__isnull=True)
#         stage_list = []
#         for s in user_stage_list:
#             try:
#                 stage = Stage.objects.get(id=s.stage.id)
#                 stage_list.append(stage)
#             except:
#                 continue



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='bee-django-mission',
    version='0.0.10',
    packages=['bee_django_mission'],
    include_package_data=True,
    license='MIT License',  # example license
    description='A simple Django app to conduct Web-based crm.',
    long_description=README,
    url='http://www.example.com/',
    author='zhangyue',
    author_email='zhangyue@zhenpuedu.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
